import os
import unittest
from .support import TempdirManager
from wmc.commands import Setup, Info, Record, Link

class BasicCommandTest(TempdirManager, unittest.TestCase):

    def get_cmd(self, cls, setup=True):
        path = os.path.join(self.mkdtemp(), 'foo')
        if setup:
            Setup(path=path).run([])

        return cls(path=path)


class TestSetup(BasicCommandTest):

    def test_main(self):
        cmd = self.get_cmd(Setup, False)
        self.assertFalse(os.path.exists(cmd.filename))
        cmd.run([])
        self.assertTrue(os.path.exists(cmd.filename))
        self.assertRaises(Exception, cmd.check)


class TestInfo(BasicCommandTest):
    def test_main(self):
        cmd = self.get_cmd(Info)
        cmd.run([])


class TestRecord(BasicCommandTest):
    def test_main(self):
        cmd = self.get_cmd(Record)
        #cmd.run([])


class TestLink(BasicCommandTest):
    def test_main(self):
        cmd = self.get_cmd(Link)
        cmd.run([])


if __name__ == '__main__':
    unittest.main()



"""Load all entry poins"""
from pkg_resources import iter_entry_points
from wmc.utils import BasicCommand


def get_basic_call():
    """get all functions from the basic class"""
    for name in dir(BasicCommand):
        if not name.startswith('_') and callable(getattr(BasicCommand, name)):
            yield name


def find_entry_points():
    """Find all register entry points."""
    entrys = [enp.name for enp in iter_entry_points(group='wmc.register_cls')]
    for name in get_basic_call():
        entrys += [enp.name for enp in iter_entry_points(group='wmc.register_{}'.format(name))]
    return entrys


def load_entry_points():
    """Load the classes and create new for only functions entry points"""
    command = {}
    for enp in iter_entry_points(group='wmc.register_cls'):
        cls = enp.load()
        command[enp.name] = cls
        command[enp.name].__name__ = enp.name

    for name in get_basic_call():
        for enp in iter_entry_points(group='wmc.register_{}'.format(name)):
            func = enp.load()
            if enp.name in command:
                setattr(command[enp.name], name, func)
            else:
                command[enp.name] = type(enp.name, (BasicCommand, ), {name: func, '__name__': enp.name})
    return command



import os
import unittest
from wmc.utils import BasicCommand


class TestBasicCommand(unittest.TestCase):

    def test_init(self):
        cmd = BasicCommand(path='/temp', file='foo.json')
        self.assertEqual('/temp', cmd.path)
        self.assertEqual('foo.json', cmd.file)
        self.assertEqual(os.path.join('/temp', 'foo.json'), cmd.filename)

        self.assertIn('name', cmd.settings)
        self.assertIn('path', cmd.settings)
        self.assertDictEqual(cmd.settings, {'name': 'temp', 'path': '/temp'})

    def test_init_defaults(self):
        cmd = BasicCommand()
        self.assertEqual(os.getcwd(), cmd.path)
        self.assertEqual('data.json', cmd.file)
        self.assertEqual(os.path.join(os.getcwd(), 'data.json'), cmd.filename)

    def test_parser(self):
        cmd = BasicCommand()
        self.assertFalse(cmd.args)
        cmd.parse_args([])
        self.assertTrue(cmd.args)

    def test_help(self):
        cmd = BasicCommand()
        self.assertIsInstance(cmd.help, str)
        self.assertEqual(cmd.help, 'The BasicCommand')

    def test_keys(self):
        cmd = BasicCommand()
        self.assertIn('name', cmd)
        self.assertIn('path', cmd)
        self.assertEqual(os.getcwd(), cmd['path'])

    def test_check(self):
        cmd = BasicCommand()
        self.assertRaises(Exception, cmd.check)

    def test_basic(self):
        cmd = BasicCommand()
        with self.assertLogs('BasicCommand', level='INFO') as cm:
            cmd.setup()
            self.assertIn('INFO:BasicCommand:Setup command', cm.output)
            cmd.create()
            self.assertIn('INFO:BasicCommand:Create settings', cm.output)
            cmd.main()
            self.assertIn('INFO:BasicCommand:Run command', cm.output)







if __name__ == '__main__':
    unittest.main()



import unittest
from wmc.dispatch import get_basic_call, find_entry_points, load_entry_points
from wmc.utils import BasicCommand

class TestDispatch(unittest.TestCase):

    def test_get_basic_call(self):
        calls = get_basic_call()
        for item in calls:
            self.assertIsInstance(item, str)
        self.assertTrue(calls)

    def test_find_entry_points(self):
        entrys = find_entry_points()
        for item in entrys:
            self.assertIsInstance(item, str)
        self.assertTrue(entrys)

    def test_load_entry_points(self):
        entrys = load_entry_points()
        for name, cls in entrys.items():
            self.assertIsInstance(name, str)
            self.assertTrue(issubclass(cls, BasicCommand))
        self.assertTrue(entrys)


if __name__ == '__main__':
    unittest.main()



"""Some tools"""
import os
import json
import logging
import argparse
from wmc import __version__


class BasicCommand():
    """The BasicCommand"""

    __version__ = __version__

    def __init__(self, path='.', file='data.json'):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.path, self.file = os.path.abspath(path), file
        self.filename = os.path.abspath(os.path.join(path, file))
        self.settings = {'name': os.path.basename(self.path), 'path': self.path}
        if os.path.isfile(self.filename):
            self.load()

        self.args = None
        self.setup()
        self.parser = self.setup_parser()

    def __getitem__(self, key):
        return self.settings.get(key)

    def __contains__(self, key):
        return key in self.settings

    def setup_parser(self):
        """Create basic parser"""
        parser = argparse.ArgumentParser(
            prog='wmc {}'.format(self.__class__.__name__),
            description='Watch me coding, a toolbox',
            epilog='Copyright 2019 AxJu | WMCv{}'.format(__version__),
        )
        parser.add_argument(
            '-V', '--version',
            action='version',
            version='%(prog)s v{}'.format(self.__version__),
        )
        return parser

    def check(self):
        """Check the project"""
        if not os.path.isfile(self.filename):
            raise Exception('No wmc-project')

    def load(self):
        """Load the data from the file"""
        with open(self.filename) as file:
            self.settings = json.load(file)

    def save(self):
        """Save the data to the file"""
        self.logger.info('Save settings (%s)', self.filename)
        with open(self.filename, 'w') as file:
            json.dump(self.settings, file, indent=4, sort_keys=True)

    def run(self, args):
        """execute the command"""
        self.check()
        self.parse_args(args)
        return self.main()

    @property
    def help(self):
        """Return __help__ ot __doc__"""
        doc = getattr(self, '__help__', None)
        if not doc:
            doc = self.__class__.__doc__ or ''
        return doc

    def parse_args(self, argv):
        """Setup parser or load some values"""
        self.logger.info('parse_args')
        self.args = self.parser.parse_args(argv)

    def setup(self):
        """Setup parser or load some values"""
        self.logger.info('Setup command')

    def create(self, **kwargs):
        """Create the data file"""
        self.logger.info('Create settings')

    def main(self, **kwargs):
        """Run the command"""
        self.logger.info('Run command')



"""The basic commands"""
import os
import sys
from datetime import datetime

import ffmpeg
from wmc.utils import BasicCommand
from wmc.dispatch import load_entry_points


class Setup(BasicCommand):
    """Setup the project"""

    __help__ = 'Some help'

    def check(self):
        """Check the project"""
        if os.path.isfile(self.filename):
            raise Exception('There are already a file')

    def main(self, **kwargs):
        super(Setup, self).main()
        self.logger.info('Start Setup path="%s"', self.path)
        if not os.path.isdir(self.path):
            os.makedirs(self.path)

        entry_points = load_entry_points()
        for cls in entry_points.values():
            cmd = cls(self.path, self.file)
            cmd.create()
            cmd.save()


class Info(BasicCommand):
    """Print some infos"""

    def main(self, **kwargs):
        """Print som infos"""
        super(Info, self).main()
        print(self.settings)


class Record(BasicCommand):
    """Start the record"""

    DATA = {
        'win': {
            "input": {
                "filename": "desktop",
                "f": "gdigrab",
                "framerate": 1,
                "video_size": (1920, 1080),
                "offset_x": -1920,
                "show_region": 1,
            },
            "output": {
                "vcodec": "libx264",
                "preset": "ultrafast",
                "r": 30,
            },
            "setpts": "N/TB/30",
        },
        'linux': {
            "input": {
                "filename": ":0.0+0,0",
                "f": "x11grab",
                "framerate": 1,
                "video_size": (1920, 1080),
            },
            "output": {
                "vcodec": "libx264",
                "preset": "ultrafast",
                "r": 30,
            },
            "setpts": "N/TB/30",
        }
    }

    def create(self, **kwargs):
        """Create the basic settings"""
        super(Record, self).create()
        platform = 'None'
        if sys.platform.startswith('win'):
            platform = 'win'
        elif sys.platform.startswith('linux'):
            platform = 'linux'
        self.settings['record'] = self.DATA.get(platform, {})

    def main(self, **kwargs):
        """Start the record"""
        super(Record, self).create()
        now = datetime.now().strftime('%Y%m%d%H%M.mp4')
        filename = os.path.join(self.settings['path'], 'video_' + now)
        settings = self.settings['record']
        stream = ffmpeg.input(**settings['input']).setpts(settings['setpts'])
        stream = ffmpeg.output(stream, filename, **settings['output'])
        ffmpeg.run(stream, overwrite_output=True)


class Link(BasicCommand):
    """Concat allvideos to one"""

    def main(self, **kwargs):
        """concat all videos to one"""
        super(Link, self).create()
        records = []
        for rec in os.listdir(self.settings['path']):
            if rec.startswith('video_'):
                records.append(os.path.join(self.settings['path'], rec))
        if not records:
            return
        records.sort()
        stream = ffmpeg.input(records[0])
        for video in records[1:]:
            stream = stream.concat(ffmpeg.input(video))
        stream = ffmpeg.output(stream, os.path.join(self.settings['path'], 'full.mp4'))
        ffmpeg.run(stream, overwrite_output=True)



"""Load all entry poins"""
import os
import sys
import logging
import argparse
from wmc import __version__
from wmc.dispatch import load_entry_points


def help_commands():
    """Print the command help."""
    commands = load_entry_points()
    for cls in commands.values():
        cmd = cls()
        text = '{:>8} v{:.5} - {}'.format(cmd.__class__.__name__, cmd.__version__, cmd.help)
        print(text)


def create_parse(commands):
    """Create the main parser"""
    parser = argparse.ArgumentParser(
        prog='wmc',
        description='Watch me coding, a toolbox',
        epilog='Copyright 2019 AxJu | WMCv{}'.format(__version__),
    )
    parser.add_argument(
        '-V', '--version',
        action='version',
        version='%(prog)s v{}'.format(__version__),
    )
    parser.add_argument(
        '-v', '--verbose', action='store_true',
        help='Enable debug infos.'
    )
    parser.add_argument(
        '-s', '--settings', default='data.json',
        help='The settings file.'
    )
    parser.add_argument(
        '-H', '--help-commands', action='store_true',
        help='Some command infos.'
    )
    parser.add_argument(
        'command', nargs='?', choices=commands,
        help='Select one command.'
    )
    parser.add_argument(
        'path', nargs='?', default=os.getcwd(),
        help='Path to the project.'
    )
    parser.add_argument(
        'args',
        help=argparse.SUPPRESS,
        nargs=argparse.REMAINDER,
    )
    return parser


def main(argv=sys.argv[1:]):
    """Create parser und run the dispatch"""
    commands = load_entry_points()
    parser = create_parse(commands.keys())
    args = parser.parse_args(argv)

    if args.verbose:
        log_format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
        logging.basicConfig(level=logging.DEBUG, format=log_format)

    if args.help_commands:
        return help_commands()

    if args.command:
        cmd = commands[args.command](args.path, args.settings)
        try:
            return cmd.run(args.args)
        except Exception as exc:
            if args.verbose:
                raise
            print('Oh no, a error :(')
            print('Error:', exc)
            print('Run with --verbose for more information.')
            return 0

    return parser.print_help()



import os
import unittest
from wmc.cli import create_parse, main


class TestCli(unittest.TestCase):

    def test_create_parse(self):
        parser = create_parse(['test'])
        args = parser.parse_args([])
        self.assertEqual(args.path, os.getcwd())
        self.assertEqual(args.settings, 'data.json')

    def test_main(self):
        main([])


if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages


setup(
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    use_scm_version=True,
    install_requires=[
        'ffmpeg-python',
    ],
    setup_requires=[
        'setuptools_scm',
    ],
    extras_require={
        # 'full':  [
        #    'wmc-resize',
        #    'lying',
        #    'blurring',
        # ],
    },
    entry_points={
        'wmc.register_cls': [
            'setup=wmc.commands:Setup',
            'info=wmc.commands:Info',
            'record=wmc.commands:Record',
            'link=wmc.commands:Link',
        ],
        'console_scripts': [
            'wmc=wmc.cli:main',
        ],
    },
)
